package liviu.tudor.catalog;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Logger;

import javax.swing.JFrame;


/**
 * Main catalog application class.
 * 
 * @author liviutudor
 */
public class Catalog 
{
	/**
	 * Used for logging -- to console by default but can be configured via Java's logging API
	 */
    private static Logger 					LOGGER = Logger.getLogger( Catalog.class.getName() );
    
    /**
     * Name of the configuration file
     */
    public static final String 				CONFIG_FILE = "jcat.properties";
    
    /**
     * Property which holds the directory we start with
     */
    public static final String 				PROP_DIR = "directory";
 
    /**
     * Program entry point
     * 
     * @param args Command-line arguments
     */
    public static void main(String[] args) 
    {
        String directory = "";
        if( (null != args) && (args.length > 1) )
            directory = args[0];
        else
            directory = readSetting();
        
        LOGGER.info( "Using directory:" + directory );
        CatalogUI panel = new CatalogUI( directory );
        JFrame frm = new JFrame( "Catalog" );
        frm.getContentPane().add( panel );
        frm.pack();
        frm.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        frm.setVisible( true );
    }
    
    /**
     * Helper function to read application setting(s) from the configuration file.
     * As with this version, the only setting we save and read is the startup directory
     * 
     * @return Startup directory if setting found in the config file, null otherwise
     * @see #CONFIG_FILE
     * @see #saveSetting(String)
     */
    protected static String readSetting()
    {
        String home = System.getProperty( "user.home" );
        if( home == null )
        {
            LOGGER.warning( "Could not read the user home property!" );
            return null;
        }
        File f = new File( home + File.separator + CONFIG_FILE );
        if( !f.exists() )
        {
            LOGGER.fine( "Properties file not found -- first run?" );
            return null;
        }
        FileInputStream fis = null;
        try
        {
            fis = new FileInputStream( f );
        }
        catch( IOException ioe )
        {
            LOGGER.warning( "I/O Error when opening file " + f.getAbsolutePath() + " -- " + ioe.getMessage() );
            return null;
        }
        Properties p = new Properties();
        try
        {
            p.load( fis );
            fis.close();
        }
        catch( IOException ioe )
        {
            LOGGER.warning( "I/O Error when reading file " + f.getAbsolutePath() + " -- " + ioe.getMessage() );
            return null;
        }
        
        return p.getProperty( PROP_DIR );
    }
    
    /**
     * Saves the application setting(s) into the configuration file
     * As with this version, the only setting we save and read is the startup directory
     * 
     * @param directory Startup directory that has been selected by the user in the GUI and therefore
     * is to be re-used at next startup
     * @return true if save successfull, false otherwise
     */
    protected static boolean saveSetting( String directory )
    {
        String home = System.getProperty( "user.home" );
        if( home == null )
        {
            LOGGER.warning( "Could not read the user home property!" );
            return false;
        }
        FileOutputStream fos = null;
        String filename = home + File.separator + CONFIG_FILE;
        try
        {
            fos = new FileOutputStream( filename );
        }
        catch( IOException ioe )
        {
            LOGGER.warning( "I/O Error when opening file " + filename + " -- " + ioe.getMessage() );
            return false;
        }
        Properties p = new Properties();
        p.setProperty( PROP_DIR, directory );
        try
        {
            p.store( fos, "Directory saved automatically." );
            fos.flush();
            fos.close();
        }
        catch( IOException ioe )
        {
            LOGGER.warning( "I/O Error when writing file " + filename + " -- " + ioe.getMessage() );
            return false;
        }
        
        return true;
    }
}
